import Globals
from Globals import Persistent, Acquisition
import AccessControl
from OFS.Folder import Folder
import DateTime
import time
from AccessControl import ClassSecurityInfo
from AccessControl.Role import RoleManager
try:
  from stripogram import html2text, html2safehtml
  _no_stripogram = 0
except:
  from Products.PythonScripts.standard import html_quote
  _no_stripogram = 1
  pass
from OFS.Cache import Cacheable
import string
import Products
from Products.ZCatalog.CatalogPathAwareness import CatalogAware
from DocumentTemplate.DT_HTML import HTML
from StructuredText.StructuredText import HTML as STXHTML
from zLOG import format_exception, LOG, ERROR, INFO
import sys
from XWorkFlow import XWorkFlow
from ZCoMIX_Common import ZCoMIX_Common

_marker = []
refresh_exc_info = {}

class XConteudo ( XWorkFlow, ZCoMIX_Common, Acquisition.Implicit, Persistent, RoleManager, CatalogAware, Cacheable, Folder ):
    """Classe que define o XConteudo."""

    friendly_name = 'Contedo'
    produtor = ""
    data_producao = "1970/01/01"
    revisor = ""
    data_revisao = "1970/01/01"
    publicador = ""
    data_publicacao = "1970/01/01"
    id_prox = ""
    data_inicial = "1970/01/01"
    data_final = "2030/01/01"
    mostrar_no_menu_superior = 0
    ordem_menu_superior = 0
    mostrar_no_menu_esquerdo = 1
    ordem_menu_esquerdo = 0
    mostrar_no_menu_direito = 0
    ordem_menu_direito = 0
    _texto = ""
    _usar_structured_text = 0
    _indexar_no_catalogo_principal = 1

    # Compatibilidade com versoes anteriores
    mostrar_no_menu = mostrar_no_menu_esquerdo
    mostrar_no_menu_principal = mostrar_no_menu_superior
    ordem_menu_principal = ordem_menu_superior
    ordem_menu_secao = ordem_menu_esquerdo

    security = ClassSecurityInfo()

    security.declareProtected('Modificar Propriedades da XPagina', 'manage_PropertiesForm')
    manage_PropertiesForm = Globals.DTMLFile("dtml/editXPaginaForm", globals(), in_ZMI=1)

    security.declareProtected('Utilizar Interface de Gerenciamento do ZCoMIX','gadgetPropertiesForm')
    gadgetPropertiesForm = Globals.DTMLFile("dtml/editXPaginaForm", globals(), in_ZMI=0)

    security.declareProtected('Utilizar Interface de Gerenciamento do ZCoMIX','manage_ZPropertiesForm')
    manage_ZPropertiesForm = Globals.DTMLFile("dtml/ZTemplate", globals(), in_ZMI=0, z_conteudo='gadgetPropertiesForm')

    security.declareProtected('Modificar Propriedades da XPagina','manage_ImagesForm')
    manage_ImagesForm = Globals.DTMLFile("dtml/gadgetImages", globals(), in_ZMI=1)

    security.declareProtected('Utilizar Interface de Gerenciamento do ZCoMIX','gadgetImagesForm')
    gadgetImagesForm = Globals.DTMLFile("dtml/gadgetImages", globals(), in_ZMI=0)

    security.declareProtected('Utilizar Interface de Gerenciamento do ZCoMIX','manage_ZImagesForm')
    manage_ZImagesForm = Globals.DTMLFile("dtml/ZTemplate", globals(), in_ZMI=0, z_conteudo='gadgetImagesForm')

    security.declareProtected('View','errorStatusPublicacao')
    errorStatusPublicacao = Globals.DTMLFile("dtml/errorStatusPublicacao", globals())

    def usar_structured_text(self):
       return self._usar_structured_text

    def __setstate__(self, state):
        XWorkFlow.__setstate__(self, state)
        if hasattr(self, 'body.htm'):
            body = getattr(self, 'body.htm')
            self._texto = body.document_src()
            self.manage_delObjects(ids='body.htm')
        if hasattr(self, 'render'):
            body = getattr(self, 'render', None)
            if hasattr(body, 'meta_type') and body.meta_type in ['DTML Document', 'DTML Method']:
              self._texto = body.document_src()
              self.manage_delObjects(ids='render')

    _properties = (
        {'id':'title','type':'string','mode':'w'},
        {'id':'produtor','type':'string','mode':'w'},
        {'id':'data_producao','type':'date','mode':'w' },
        {'id':'revisor','type':'string','mode':'w'},
        {'id':'data_revisao','type':'date','mode':'w' },
        {'id':'publicador','type':'string','mode':'w'},
        {'id':'data_publicacao','type':'date','mode':'w' },
        {'id':'data_inicial','type':'date','mode':'w' },
        {'id':'data_final','type':'date','mode':'w' },
        {'id':'mostrar_no_menu_esquerdo','type':'boolean','mode':'w' },
        {'id':'ordem_menu_esquerdo','type':'boolean','mode':'w' },
        {'id':'mostrar_no_menu_direito','type':'boolean','mode':'w' },
        {'id':'ordem_menu_direito','type':'boolean','mode':'w' },
        )

    _cache_namespace_keys=()

    def all_meta_types(self):
        """ What can you put inside me? """
        f = lambda x: x['name'] in ('DTML Method', 'DTML Document', 'Image', 'Z SQL Method',  'Script (Python)',
                                                'Folder', 'File', 'Formulator Form', 'XLink')
        return filter(f, Products.meta_types)

    security.declareProtected('View', 'getTexto')
    def getTexto(self, REQUEST=None):
        "Retorna o conteudo do objeto como Texto."
        result = "%s" % self._texto
        return result

    def manage_options(self, REQUEST=None):
        manage_options=((
            {'label':'Propriedades', 'action':'manage_PropertiesForm'},
            {'label':'Visualizar', 'action':''},
            {'label':'Imagens', 'action':'manage_ImagesForm'},
            )
            + Folder.manage_options[:1]
            + Folder.manage_options[3:]
            + Cacheable.manage_options)

        zcomix_manage_options=((
            {'label':'Propriedades', 'action':'manage_ZPropertiesForm'},
            {'label':'Imagens', 'action':'manage_ZImagesForm'},
            {'label':'Visualizar', 'action':''},
            ))

        if self.SecurityGetUser().has_permission('View management screens', self):
          return manage_options
        else:
          return zcomix_manage_options


    meta_type = 'XConteudo'

    def __init__(self, id, title):
        self.id = id
        self.title = title

    def PrincipiaSearchSource(self):
	"""Suporte para busca."""
	return self.getTexto()

    def document_src(self, REQUEST=None, RESPONSE=None):
	"""Retorna o source do documento."""
	if RESPONSE is not None:
	   RESPONSE.setHeader('Content-Type', 'text/plain')
	return self.getTexto()

    security.declareProtected('View', 'render')
    def render(self, REQUEST=None, *args, **kw):
        "Retorna o conteudo do objeto como HTML."
        if not self._cache_namespace_keys:
            data = self.ZCacheable_get(default=_marker)
            if data is not _marker:
                # Return cached results.
                return data

        if REQUEST is None:
          REQUEST = self.REQUEST

        kw['document_id']    = self.getId()
        kw['document_title'] = self.title

        if self.checkPublished():
          body = self.getTexto()
          if self._usar_structured_text:
            r=STXHTML(body, header=0)
          else:
            r=apply(HTML(body), (self, REQUEST), kw)
        else:
          return self.errorStatusPublicacao(REQUEST)
        if not self._cache_namespace_keys:
          self.ZCacheable_set(r)
        return r

    def __call__(self, REQUEST=None, *args, **kw):
        if REQUEST is None:
          REQUEST = self.REQUEST
        return apply(self.render, (self, REQUEST), kw)

    def __str__(self, REQUEST=None, *args, **kw):
        if REQUEST is None:
          REQUEST = self.REQUEST
        return self._texto

    def logError(productid='XConteudo'):
        exc = sys.exc_info()
        try:
            LOG('Render', ERROR, 'Excecao enquanto renderizava: \n %s'
                % productid, error=exc)
        finally:
            exc = None

    security.declareProtected('View', 'long_title')
    def long_title(self, format='html', REQUEST=None):
        "Titulo da Secao + Titulo da Pagina"
        if format == 'html':
                title = self.title_or_id()
                parent = self.aq_parent
                while parent.meta_type!='XSite':
                        if parent.meta_type=='XSecao':
                          try: title = parent.title_or_id() + "&nbsp;&raquo;&nbsp;" + title
                          except: pass
                        parent = parent.aq_parent
                return title
        title = self.aq_parent.title_or_id() + " : " + self.title_or_id()
        parent = self.aq_parent
        while parent.meta_type!='XSite':
                if parent.meta_type=='XSecao':
                        title = parent.title_or_id() + " : " + title
                parent = parent.aq_parent
        return title

    security.declareProtected('View', 'summary')
    def summary(self, REQUEST=None):
        "Resumo do Texto Pagina"
        txtresumo = ""
        try:
          txtresumo = self.render(REQUEST=REQUEST)
        except:
          self.logError()
          pass

        if txtresumo is not None:
          if _no_stripogram:
            txtresumo =  html_quote(txtresumo)
          else:
            txtresumo = html2text(txtresumo)
          pos  = string.rfind(txtresumo, '\n', 0, 1000)
          txtresumo = txtresumo[:pos]
        else:
           txtresumo = self.title
        return txtresumo

    security.declareProtected('View', 'Caminho')
    def Caminho(self, REQUEST=None):
        """Caminho ate a raiz"""
        result = []
        req = None

        if REQUEST is None:
          REQUEST = self.REQUEST

        if REQUEST is not None:
                req = REQUEST

        if not req and self.REQUEST is not None:
                req = self.REQUEST

        if not req:
                return result

        i = 0;
        for node in req.PARENTS:
          try:
            if hasattr(node, 'meta_type') and (node.meta_type != 'XTemplate' or i==0):
              result.append(node)
              if node.meta_type == 'XSite':
                break
          except:
            break
          i += 1
        return result

    security.declareProtected('Modificar Propriedades do XConteudo','manage_editXConteudo')
    def manage_editXConteudo(self,  REQUEST=None):
        """ Modifica as Propriedades do XConteudo """

Globals.default__class_init__(XConteudo)

def manage_addXConteudo(self, REQUEST=None):
        """ Adiciona um XConteudo """

manage_addXConteudoForm = Globals.HTMLFile('dtml/addXConteudoForm', globals())
