from Globals import InitializeClass
from Globals import Persistent, Acquisition
import Globals
import AccessControl
import OFS
from OFS.Cache import Cacheable
from Products.ZCatalog.CatalogPathAwareness import CatalogAware
from AccessControl import ClassSecurityInfo
from XWorkFlow import XWorkFlow
from ZCoMIX_Common import ZCoMIX_Common

class XLink ( XWorkFlow, ZCoMIX_Common, Persistent, Acquisition.Implicit, CatalogAware, Cacheable, OFS.SimpleItem.Item ):
    """Classe que define o XLink."""

    friendly_name = 'Link'
    title = None
    abs_url = None
    data_inicial = '1970/01/01'
    data_final = '2030/01/01'
    link_target='_self'
    estilo_css = None
    ordem_layout = 0
    mostrar_no_menu_esquerdo = 1
    mostrar_no_menu_superior = 0
    mostrar_no_menu_direito = 0
    ordem_menu_superior = 0
    ordem_menu_esquerdo = 0
    ordem_menu_direito=0
    default_targets = [('Nova Janela','_blank'), ('Mesma Janela','_self')]
    security = ClassSecurityInfo()
    meta_type = 'XLink'

    # Compatibilidade com versoes anteriores
    mostrar_no_menu = mostrar_no_menu_esquerdo
    mostrar_no_menu_principal = mostrar_no_menu_superior
    ordem_menu_principal = ordem_menu_superior
    ordem_menu_secao = ordem_menu_esquerdo

    _properties = (
        {'id':'title','type':'string','mode':'w'},
        {'id':'data_inicial','type':'date','mode':'w' },
        {'id':'data_final','type':'date','mode':'w' },
        {'id':'link_target','type':'string','mode':'w' },
        {'id':'estilo_css','type':'string','mode':'w' },
        {'id':'ordem_layout','type':'int','mode':'w' },
        {'id':'mostrar_no_menu_superior','type':'boolean','mode':'w' },
        {'id':'mostrar_no_menu_esquerdo','type':'boolean','mode':'w' },
        {'id':'mostrar_no_menu_direito','type':'boolean','mode':'w' },
        {'id':'ordem_menu_superior','type':'boolean','mode':'w' },
        {'id':'ordem_menu_esquerdo','type':'boolean','mode':'w' },
        {'id':'ordem_menu_direito','type':'boolean','mode':'w' },
        )

    security.declareProtected('Modificar Propriedades do XLink', 'manage_PropertiesForm')
    manage_PropertiesForm = Globals.DTMLFile("dtml/editXLinkForm", globals(), in_ZMI=1)

    security.declareProtected('Utilizar Interface de Gerenciamento do ZCoMIX', 'gadgetPropertiesForm')
    gadgetPropertiesForm = Globals.DTMLFile("dtml/editXLinkForm", globals(), in_ZMI=0)

    security.declareProtected('Utilizar Interface de Gerenciamento do ZCoMIX','manage_ZPropertiesForm')
    manage_ZPropertiesForm = Globals.DTMLFile("dtml/ZTemplate", globals(), in_ZMI=0, z_conteudo='gadgetPropertiesForm')

    def manage_options(self, REQUEST=None):
      manage_options=((
          {'label':'Propriedades', 'action':'manage_PropertiesForm'},
          {'label':'Visualizar', 'action':'manage_testaXLink'},
          ) + OFS.SimpleItem.Item.manage_options)

      zcomix_manage_options=((
          {'label':'Propriedades', 'action':'manage_ZPropertiesForm'},
          {'label':'Visualizar', 'action':'manage_testaXLink'},
          ))

      if self.SecurityGetUser().has_permission('View management screens', self):
        return manage_options
      else:
        return zcomix_manage_options

    def __setstate__(self, state):
        XWorkFlow.__setstate__(self, state)
        if self.data_final == '2100/01/01':
            self.data_final = '2030/01/01'

    def __init__(self, id, title=None, url=None):
        self.id = id
        self.title = title
        self.abs_url = url


    def PrincipiaSearchSource(self):
	"""Suporte para busca."""
	return "%s (%s)" % (self.title_or_id(), self.absolute_url())

    security.declareProtected('View','title_or_id')
    def title_or_id(self, REQUEST=None):
        if self.title is not None:
                return self.title
        else:
                return self.id

    security.declareProtected('View','absolute_url')
    def absolute_url(self, REQUEST = None, relative = 0):
        if self.abs_url is not None:
                return self.abs_url
        else:
                return "#"

    security.declareProtected('View', 'render')
    def render(self, REQUEST=None):
        "Retorna o conteudo do objeto como HTML."
        result = ""
        if self.estilo_css is None:
                result += "<a href='%s' target='%s'>%s</a>" % (self.absolute_url(), self.link_target, self.title_or_id())
        else:
                result += "<a class='%s' href='%s' target='%s'>%s</a>" % (self.estilo_css, self.absolute_url(), self.link_target, self.title_or_id())
        return result

    security.declareProtected('View', 'manage_testaXLink')
    def manage_testaXLink(self, REQUEST=None):
        "Retorna o conteudo do objeto como HTML. Utilizado na Interface do Zope."

        if self.aq_parent.meta_type=='XPaginaLayout':
                css = self.aq_parent.estilo_css
        else:
                css = self.estilo_css

        result = "<html><HEAD>"
        result += "<LINK REL=StyleSheet HREF='%s' TYPE='text/css'>" % css
        result += "</HEAD><BODY>"

        result += self.render()
        result += "</BODY></HTML>"
        return result

    security.declareProtected('Modificar Propriedades do XLink', 'manage_Properties')
    def manage_Properties(self, title=None, url=None,
        mostrar_no_menu_superior=0, mostrar_no_menu_esquerdo=0, mostrar_no_menu_direito=0,
        data_inicial='1970/01/01', data_final='2100/01/01',
        link_target='_self', estilo_css=None, in_ZMI=1, REQUEST=None, *args, **kw):
        "Modifica as Propriedades da XLink"
        self.title = title
        self.abs_url = url
        try: self.data_inicial = DateTime.DateTime(data_inicial).ISO()
        except: self.data_inicial = '1970/01/01'
        try: self.data_final = DateTime.DateTime(data_final).ISO()
        except: self.data_final = '2030/01/01'
        self.link_target=link_target
        self.mostrar_no_menu_superior = mostrar_no_menu_superior
        self.mostrar_no_menu_esquerdo = mostrar_no_menu_esquerdo
        self.mostrar_no_menu_direito = mostrar_no_menu_direito
        self.estilo_css = estilo_css
        if REQUEST is not None:
          if in_ZMI:
            return self.manage_PropertiesForm(REQUEST, management_view='Propriedades',
                manage_tabs_message='O Link <strong>%s</strong> foi editado.' % self.id
                )
          else:
            return self.manage_ZPropertiesForm(REQUEST, management_view='Propriedades',
                manage_tabs_message='O Link <strong>%s</strong> foi editado.' % self.id
                )

InitializeClass(XLink)

def manage_addXLink(self, id, title=None, url=None,
        mostrar_no_menu_superior=0, mostrar_no_menu_esquerdo=0, mostrar_no_menu_direito=0,
        data_inicial='1970/01/01', data_final='2100/01/01',
        link_target='_self', estilo_css=None, REQUEST=None):
        """Cria um XLink e o adiciona ao seu parent Folder.
        O argumento 'self' referencia o parent Folder.
        """
        link = XLink(id, title, url)
        try: link.data_inicial = DateTime.DateTime(data_inicial).ISO()
        except: link.data_inicial = '1970/01/01'
        try: link.data_final = DateTime.DateTime(data_final).ISO()
        except: link.data_final = '2030/01/01'
        link.mostrar_no_menu_superior = mostrar_no_menu_superior
        link.mostrar_no_menu_esquerdo = mostrar_no_menu_esquerdo
        link.mostrar_no_menu_direito = mostrar_no_menu_direito
        link.link_target = link_target
        link.estilo_css = estilo_css
        self._setObject(id, link)
        if REQUEST is not None:
                return self.manage_main(self, REQUEST)

manage_addXLinkForm = Globals.HTMLFile('dtml/addXLinkForm', globals())
