import XConteudo
import OFS
import Globals
import time
import DateTime
from AccessControl import ClassSecurityInfo
import XParagrafo
import string
import Products
from DocumentTemplate.DT_HTML import HTML
from StructuredText.StructuredText import HTML as STXHTML
from OFS.Folder import Folder
from OFS.Cache import Cacheable

_marker = []

class XPaginaLayout ( XConteudo.XConteudo ):
    """Classe que define a XPaginaLayout."""

    estilo_css = None
    friendly_name = 'Pgina com Layout'
    meta_type = 'XPaginaLayout'

    security = ClassSecurityInfo()

    security.declareProtected('Modificar Propriedades da XPaginaLayout', 'manage_PropertiesForm')
    manage_PropertiesForm = Globals.DTMLFile("dtml/editXPaginaLayoutForm", globals(), in_ZMI=1)

    security.declareProtected('Modificar Propriedades da XPaginaLayout','manage_OrdemForm')
    manage_OrdemForm = Globals.DTMLFile("dtml/editXPaginaLayoutOrdemForm", globals(), in_ZMI=1)

    security.declareProtected('Utilizar Interface de Gerenciamento do ZCoMIX', 'gadgetPropertiesForm')
    gadgetPropertiesForm = Globals.DTMLFile("dtml/editXPaginaLayoutForm", globals(), in_ZMI=0)

    security.declareProtected('Utilizar Interface de Gerenciamento do ZCoMIX','gadgetOrdemForm')
    gadgetOrdemForm = Globals.DTMLFile("dtml/editXPaginaLayoutOrdemForm", globals(), in_ZMI=0)

    security.declareProtected('Utilizar Interface de Gerenciamento do ZCoMIX','gadgetContentsForm')
    gadgetContentsForm = Globals.DTMLFile("dtml/gadgetContents", globals(), in_ZMI=0)

    security.declareProtected('Utilizar Interface de Gerenciamento do ZCoMIX','manage_ZPropertiesForm')
    manage_ZPropertiesForm = Globals.DTMLFile("dtml/ZTemplate", globals(), in_ZMI=0, z_conteudo='gadgetPropertiesForm')

    security.declareProtected('Utilizar Interface de Gerenciamento do ZCoMIX','manage_ZOrdemForm')
    manage_ZOrdemForm = Globals.DTMLFile("dtml/ZTemplate", globals(), in_ZMI=0, z_conteudo='gadgetOrdemForm')

    security.declareProtected('Utilizar Interface de Gerenciamento do ZCoMIX','manage_ZContentsForm')
    manage_ZContentsForm = Globals.DTMLFile("dtml/ZTemplate", globals(), in_ZMI=0, z_conteudo='gadgetContentsForm')

    def __init__(self, id, title):
        self.id = id
        self.title = title

    def manage_options(self, REQUEST=None):
      manage_options=((
          {'label':'Propriedades', 'action':'manage_PropertiesForm'},)
          + Folder.manage_options[:1] +
          ({'label':'Layout', 'action':'manage_OrdemForm'},
          {'label':'HTML', 'action':'manage_testaXPaginaLayout'},)
          + Folder.manage_options[3:]
          + Cacheable.manage_options
          )

      zcomix_manage_options=((
          {'label':'Conteudo', 'action':'manage_ZContentsForm'},
          {'label':'Propriedades', 'action':'manage_ZPropertiesForm'},
          {'label':'Layout', 'action':'manage_ZOrdemForm'},
          {'label':'Visualizar', 'action':''},))

      if self.SecurityGetUser().has_permission('View management screens', self):
        return manage_options
      else:
        return zcomix_manage_options

    def all_meta_types(self):
        """ What can you put inside me? """
        f = lambda x: x['name'] in ('DTML Method', 'DTML Document', 'Image', 'Z SQL Method',  'Script (Python)',
                                                'Folder', 'File', 'Formulator Form', 'XLink', 'XParagrafo', 'XImagem')
        return filter(f, Products.meta_types)

    security.declareProtected('View', 'render')
    def render(self, REQUEST=None, **kw):
        "Renderiza o conteudo do objeto como HTML."
        if not self._cache_namespace_keys:
            data = self.ZCacheable_get(default=_marker)
            if data is not _marker:
                # Return cached results.
                return data

        if REQUEST is None:
                REQUEST = self.REQUEST

        result = ""

        lista_objetos = self.objectValues()
        lista_objetos.sort(lambda x, y: cmp(x.ordem_layout, y.ordem_layout))
        for obj in lista_objetos:
                result += obj.render(REQUEST=REQUEST)

        if self._usar_structured_text:
          result = STXHTML(result, header=0)
        else:
          result=HTML(result,REQUEST=REQUEST,kw=kw)(self)

        if not self._cache_namespace_keys:
            self.ZCacheable_set(result)
        return result

    security.declareProtected('View', 'manage_testaXPaginaLayout')
    def manage_testaXPaginaLayout(self, REQUEST=None):
        "Retorna o conteudo do objeto como HTML. Utilizado na Interface do Zope."
        result = "<html><HEAD>"
        result += "<TITLE>%s</TITLE>"
        result += "<LINK REL=StyleSheet HREF='%s' TYPE='text/css'>" % self.estilo_css
        result += "</HEAD><BODY>"
        result += self.render(REQUEST=REQUEST)
        result += "</BODY></HTML>"
        return result

    security.declareProtected('Access contents information', 'estilos_css')
    def estilos_css(self, tag='', REQUEST=None):
        "Retorna uma lista com os estilos disponiveis no arquivo de estilos selecionado."
        if self.estilo_css is None:
                return

        estilo = getattr(self, self.estilo_css, None)
        if estilo is None:
                tpl = getattr(self, self.getTemplate(REQUEST), None)
                estilo = getattr(tpl, self.estilo_css, None)
        if estilo is None:
                tpl = getattr(self, self.template_principal, None)
                estilo = getattr(tpl, self.estilo_css, None)

        if estilo is None:
                return ""

        texto = string.split(estilo.style(), '\n')
        results = []
        j = 0
        for line in texto:
                parts = string.split(line, ' {')
                style = parts[0]
                if ((style != '') and (style[0] == '.')) or ((string.find(style, tag+'.')==0)):
                        style = string.replace(style, tag+'.', '')
                        style = string.replace(style, '.', '')
                        results.append(style)
        return results

    security.declareProtected('Modificar Propriedades da XPaginaLayout','manage_Ordem')
    def manage_Ordem(self, items_layout, in_ZMI=1, REQUEST=None, *args, **kw):
        "Modifica a ordem dos objetos no Layout"
        ordem = 0
        for i in items_layout:
                ob = getattr(self, i, None)
                if ob is not None:
                        ob.ordem_layout = ordem
                ordem += 1
        self.ZCacheable_invalidate()
        if REQUEST is not None:
          if in_ZMI:
            return self.manage_OrdemForm(REQUEST, management_view='Propriedades',
                manage_tabs_message='A Pagina (Layout) <strong>%s</strong> foi editada.' % self.id
                )
          else:
            return self.manage_ZOrdemForm(REQUEST, management_view='Propriedades',
                manage_tabs_message='A Pagina (Layout) <strong>%s</strong> foi editada.' % self.id
                )

    security.declareProtected('Modificar Propriedades da XPaginaLayout', 'manage_Properties')
    def manage_Properties(self, title, data_inicial='1970/01/01', data_final='2030/01/01',
    mostrar_no_menu_superior = 0, mostrar_no_menu_esquerdo = 0, mostrar_no_menu_direito = 0,
    estilo_css=None, usar_structured_text=0, in_ZMI=1, REQUEST=None, *args, **kw):
        "Modifica as Propriedades da XPaginaLayout"
        self.title = title
        try: self.data_inicial = DateTime.DateTime(data_inicial).ISO()
        except: self.data_inicial = '1970/01/01'
        try: self.data_final = DateTime.DateTime(data_final).ISO()
        except: self.data_final = '2030/01/01'
        self.mostrar_no_menu_superior = mostrar_no_menu_superior
        self.mostrar_no_menu_esquerdo = mostrar_no_menu_esquerdo
        self.mostrar_no_menu_direito = mostrar_no_menu_direito
        self.estilo_css = estilo_css
        self._usar_structured_text = usar_structured_text
        if self.catalog_principal is not None:
                self.manage_editCataloger(self.catalog_principal)
        self.reindex_object()
        self.ZCacheable_invalidate()

        if REQUEST is not None:
          if in_ZMI:
            return self.manage_PropertiesForm(REQUEST, management_view='Propriedades',
                manage_tabs_message='A Pagina (Layout) <strong>%s</strong> foi editada.' % self.id
                )
          else:
            return self.manage_ZPropertiesForm(REQUEST, management_view='Propriedades',
                manage_tabs_message='A Pagina (Layout) <strong>%s</strong> foi editada.' % self.id
                )

Globals.default__class_init__(XPaginaLayout)

def manage_addXPaginaLayout(self, id, title, produtor='admin', data_producao='1970/01/01', \
        data_inicial='1970/01/01', data_final='2030/01/01', mostrar_no_menu_superior=0,
        mostrar_no_menu_esquerdo=0, mostrar_no_menu_direito=0, estilo_css=None,
		usar_structured_text=0, REQUEST=None):
        """Cria uma XPaginaLayout e o adiciona ao seu parent Folder.
        O argumento 'self' referencia o parent Folder.
        """
        pagina = XPaginaLayout(id, title)
        pagina.produtor = REQUEST.AUTHENTICATED_USER.getId()
        t = time.time()
        pagina.data_producao = DateTime.DateTime(t).ISO()
        try: pagina.data_inicial = DateTime.DateTime(data_inicial).ISO()
        except: pagina.data_inicial = '1970/01/01'
        try: pagina.data_final = DateTime.DateTime(data_final).ISO()
        except: pagina.data_final = '2030/01/01'
        pagina.mostrar_no_menu_superior = mostrar_no_menu_superior
        pagina.mostrar_no_menu_esquerdo = mostrar_no_menu_esquerdo
        pagina.mostrar_no_menu_direito = mostrar_no_menu_direito
        pagina._usar_structured_text = usar_structured_text
        pagina.estilo_css = estilo_css
        self._setObject(id, pagina)
        pagina = getattr(self, id)
        if self.catalog_principal is not None:
                pagina.manage_editCataloger(self.catalog_principal)
                getattr(self, self.catalog_principal).catalog_object(pagina, '/' + pagina.absolute_url(relative=1))


        if REQUEST is not None:
                return self.manage_main(self, REQUEST)

manage_addXPaginaLayoutForm = Globals.HTMLFile('dtml/addXPaginaLayoutForm', globals())
