import Globals
from Globals import Persistent, Acquisition
import AccessControl
import OFS
from AccessControl import ClassSecurityInfo, SecurityManagement
from AccessControl.Role import RoleManager
from XLink import XLink
import Products
from XConteudo import XConteudo
from Products.ZCatalog.CatalogPathAwareness import CatalogAware
from XWorkFlow import XWorkFlow
import DateTime
import time
from DocumentTemplate.DT_HTML import HTML
from StructuredText.StructuredText import HTML as STXHTML
from XWorkFlow import dt2us
from ZCoMIX_Common import ZCoMIX_Common

class XSecao ( XWorkFlow, ZCoMIX_Common, Acquisition.Implicit, Persistent, RoleManager, CatalogAware, OFS.Folder.Folder ):
    """Classe que define a XSecao."""

    friendly_name = 'Seo'
    responsavel = ''
    email_responsavel = ''
    responsavel_revisao = ''
    responsavel_publicacao = ''
    monitorada = 0
    data_inicial = '1970/01/01'
    data_final = '2030/01/01'
    pagina_inicial = None
    template = None
    mostrar_no_menu_superior = 0
    mostrar_no_menu_esquerdo = 1
    mostrar_no_menu_direito = 0
    ordem_menu_superior = 0
    ordem_menu_esquerdo = 0
    ordem_menu_direito = 0
    usar_menu_esquerdo_pai = 0
    usar_menu_direito_pai = 0
    property_extensible_schema__ = 1

    # Compatibilidade com versoes anteriores
    mostrar_no_menu = mostrar_no_menu_esquerdo
    mostrar_no_menu_principal = mostrar_no_menu_superior
    ordem_menu_principal = ordem_menu_superior
    ordem_menu_secao = ordem_menu_esquerdo
    usar_menu_pai = usar_menu_esquerdo_pai

    _properties = (
        {'id':'title','type':'string','mode':'w'},
        {'id':'responsavel','type':'string','mode':'w'},
        {'id':'email_responsavel','type':'string','mode':'w'},
        {'id':'responsavel_revisao','type':'string','mode':'w'},
        {'id':'responsavel_publicacao','type':'string','mode':'w'},
        {'id':'monitorada','type':'boolean','mode':'w'},
        {'id':'data_inicial','type':'date','mode':'w' },
        {'id':'data_final','type':'date','mode':'w' },
        {'id':'pagina_inicial','type':'string','mode':'w' },
        {'id':'template','type':'string','mode':'w' },
        {'id':'mostrar_no_menu_superior','type':'boolean','mode':'w' },
        {'id':'mostrar_no_menu_esquerdo','type':'boolean','mode':'w' },
        {'id':'mostrar_no_menu_direito','type':'boolean','mode':'w' },
        {'id':'ordem_menu_superior','type':'boolean','mode':'w' },
        {'id':'ordem_menu_esquerdo','type':'boolean','mode':'w' },
        {'id':'ordem_menu_direito','type':'boolean','mode':'w' },
        {'id':'usar_menu_esquerdo_pai','type':'boolean','mode':'w' },
        {'id':'usar_menu_direito_pai','type':'boolean','mode':'w' },
        )

    meta_type = 'XSecao'
    security = ClassSecurityInfo()

    security.declareProtected('Modificar Propriedades da XSecao', 'manage_PropertiesForm')
    manage_PropertiesForm = Globals.DTMLFile("dtml/editXSecaoForm", globals(), in_ZMI=1)

    security.declareProtected('Utilizar Interface de Gerenciamento do ZCoMIX','gadgetPropertiesForm')
    gadgetPropertiesForm = Globals.DTMLFile("dtml/editXSecaoForm", globals(), in_ZMI=0)

    security.declareProtected('Modificar Propriedades da XSecao', 'manage_MenuForm')
    manage_MenuForm = Globals.DTMLFile("dtml/editXSecaoMenuForm", globals(), in_ZMI=1)

    security.declareProtected('Utilizar Interface de Gerenciamento do ZCoMIX','gadgetMenuForm')
    gadgetMenuForm = Globals.DTMLFile("dtml/editXSecaoMenuForm", globals(), in_ZMI=0)

    security.declareProtected('Utilizar Interface de Gerenciamento do ZCoMIX','manage_ZPropertiesForm')
    manage_ZPropertiesForm = Globals.DTMLFile("dtml/ZTemplate", globals(), in_ZMI=0, z_conteudo='gadgetPropertiesForm')

    security.declareProtected('Utilizar Interface de Gerenciamento do ZCoMIX','manage_ZMenuForm')
    manage_ZMenuForm = Globals.DTMLFile("dtml/ZTemplate", globals(), in_ZMI=0, z_conteudo='gadgetMenuForm')

    security.declareProtected('View','errorTemplatePrincipal')
    errorTemplatePrincipal = Globals.DTMLFile("dtml/errorTemplatePrincipal", globals())

    def __setstate__(self, state):
        XWorkFlow.__setstate__(self, state)

    def all_meta_types(self):
        """ What can you put inside me? """
        f = lambda x: x['name'] in ('DTML Method', 'DTML Document', 'Image', 'Z SQL Method',  'Script (Python)',  'LDAPLoginAdapter', 'LDAPUserManager', 'MySQL UserFolder',
                                                'Folder', 'File', 'Formulator Form', 'XLink', 'XParagrafo', 'XImagem', 'XPagina', 'XPaginaLayout', 'XSecao', 'XNoticia', 'Z MySQL Database Connection')
        return filter(f, Products.meta_types)

    def manage_options(self, REQUEST=None):
      manage_options = ( OFS.Folder.Folder.manage_options[:1] + (
          {'label':'Propriedades', 'action':'manage_PropertiesForm'},
          {'label':'Menu', 'action':'manage_MenuForm'},
          {'label':'Visualizar', 'action': '' },
          ) + OFS.Folder.Folder.manage_options[3:] )

      zcomix_manage_options = ((
          {'label':'Contedo', 'action':'manage_ZWorkspaceForm'},
          {'label':'Propriedades', 'action':'manage_ZPropertiesForm'},
          {'label':'Menu', 'action':'manage_ZMenuForm'},
          {'label':'Visualizar', 'action': '' },
          ))

      if self.SecurityGetUser().has_permission('View management screens', self):
        return manage_options
      else:
        return zcomix_manage_options

    def __init__(self, id, title):
        self.id = id
        self.title = title

    Caminho = XConteudo.Caminho

    security.declareProtected('View', 'index_html')
    def index_html(self, REQUEST=None, *args, **kw):
        "Visualizacao principal (pagina inicial) da XSecao"
        if self.restrictedTraverse(REQUEST.PATH_INFO) == self:
                url = self.absolute_url()
                url += '/' + self.getPaginaInicial(REQUEST)
                return REQUEST.RESPONSE.redirect(url)
        pg = self.restrictedTraverse(REQUEST.PATH_INFO)
        pg = pg.__of__(self)
        tmpl = self.getTemplate()
        if tmpl is None:
          return self.errorTemplatePrincipal(REQUEST)
        tmpl = getattr(self, tmpl)
        index = getattr(tmpl, 'index_html')
        if hasattr(index, '__of__'):
                index = index.__of__(pg)
                return apply(index, (pg, REQUEST), kw)

    security.declarePublic('View', 'getTemplate')
    def getTemplate(self, REQUEST=None):
        "Retorna o Template, caso existir ou o Template Padrao"

        if hasattr(self, 'template') and self.template is not None:
          return self.template
        if self.aq_parent.meta_type=='XSite':
          return self.getTemplatePrincipal()
        if hasattr(self.aq_parent, 'getTemplate'):
          return self.aq_parent.getTemplate()
        return self.getTemplatePrincipal()

    security.declareProtected('View', 'getPaginaInicial')
    def getPaginaInicial(self, REQUEST=None):
        "Retorna a pagina inicial, caso existir ou o primeiro XPagina, ou a primeira XSecao"
        pgini = self.pagina_inicial
        if not pgini:
                for pg in self.objectValues(['XPagina']):
                        pgini = pg.getId()
                        break

        if not pgini:
                for pg in self.objectValues(['XSecao']):
                        pgini = pg.getId()
                        break

        if not pgini:
                pgini = 'index_html'

        return pgini

    security.declareProtected('View', 'menuItems')
    def menuItems(self, REQUEST=None):
        "Lista de items a serem mostrados no Menu Esquerdo da XSecao"
        return self.itensMenu(esquerdo=1, REQUEST=REQUEST)

    security.declareProtected('View', 'itensMenuEsquerdo')
    def itensMenuEsquerdo(self, REQUEST=None):
        "Lista de items a serem mostrados no Menu Esquerdo da XSecao"
        return self.itensMenu(esquerdo=1, REQUEST=REQUEST)

    security.declareProtected('View', 'itensMenuDireito')
    def itensMenuDireito(self, REQUEST=None):
        "Lista de items a serem mostrados no Menu Direito da XSecao"
        return self.itensMenu(esquerdo=0, REQUEST=REQUEST)


    security.declareProtected('Access contents information', 'itensMenu')
    def itensMenu(self, esquerdo=1, REQUEST=None):
        "Lista de items a serem mostrados no Menu da XSecao"
        items = []
        secao = self
        if esquerdo:
          if hasattr(self, 'usar_menu_esquerdo_pai') and self.usar_menu_esquerdo_pai:
            if self.aq_parent.meta_type=='XSite':
              secao = self.restrictedTraverse(self.getSecaoPrincipal())
            else:
              secao = self.aq_parent
              return secao.itensMenu(esquerdo=esquerdo, REQUEST=REQUEST)
          for item in secao.objectValues(['XPagina', 'XPaginaLayout', 'XSecao', 'XLink']):
              if item.mostrar_no_menu_esquerdo and item.checkPublished():
                items.append(item)
        else:
          if hasattr(self, 'usar_menu_direito_pai') and self.usar_menu_direito_pai:
            if self.aq_parent.meta_type=='XSite':
              secao = self.restrictedTraverse(self.getSecaoPrincipal())
            else:
              secao = self.aq_parent
              return secao.itensMenu(esquerdo=esquerdo, REQUEST=REQUEST)
          for item in secao.objectValues(['XPagina', 'XPaginaLayout', 'XSecao', 'XLink']):
              if item.mostrar_no_menu_direito and item.checkPublished():
                items.append(item)

        if len(items)>1:
          if esquerdo:
            if items[0].ordem_menu_esquerdo == items[1].ordem_menu_esquerdo:
              items.sort(lambda x, y: cmp(x.title_or_id(), y.title_or_id()))
            else:
              items.sort(lambda x, y: cmp(x.ordem_menu_esquerdo, y.ordem_menu_esquerdo))
          else:
            if items[0].ordem_menu_direito == items[1].ordem_menu_direito:
              items.sort(lambda x, y: cmp(x.title_or_id(), y.title_or_id()))
            else:
              items.sort(lambda x, y: cmp(x.ordem_menu_direito, y.ordem_menu_direito))
        return items

    security.declareProtected('Modificar Propriedades da XSecao','manage_changeUsarPai')
    def manage_changeUsarPai(self, usar_menu_pai=0, esquerdo=0, in_ZMI=1, REQUEST=None, *args, **kw):
        "Utilizar menu da secao Pai"
        if esquerdo:
                self.usar_menu_esquerdo_pai = usar_menu_pai
        else:
                self.usar_menu_direito_pai = usar_menu_pai
        if in_ZMI:
          return self.manage_MenuForm(REQUEST, management_view = 'Menu',
                        manage_tabs_message='A Secao <strong>%s</strong> foi editada.' % self.id)
        else:
          return self.manage_ZMenuForm(REQUEST, management_view = 'Menu',
                        manage_tabs_message='A Secao <strong>%s</strong> foi editada.' % self.id)

    security.declareProtected('Modificar Propriedades da XSecao','manage_Menu')
    def manage_Menu(self, menu_items=[], esquerdo=0, in_ZMI=1, REQUEST=None, *args, **kw):
        "Modifica a ordem dos Menus da XSecao"
        ordem = 0
        for i in menu_items:
                ob = getattr(self, i, None)
                if ob is not None:
                        if esquerdo:
                                ob.ordem_menu_esquerdo = ordem
                        else:
                                ob.ordem_menu_direito = ordem
                ordem += 1

        if REQUEST is not None:
          if in_ZMI:
                return self.manage_MenuForm(REQUEST, management_view = 'Menu',
                        manage_tabs_message='A Secao <strong>%s</strong> foi editada.' % self.id)
          else:
                return self.manage_ZMenuForm(REQUEST, management_view = 'Menu',
                        manage_tabs_message='A Secao <strong>%s</strong> foi editada.' % self.id)

    security.declareProtected('Modificar Propriedades da XSecao', 'manage_Properties')
    def manage_Properties(self, title, responsavel='admin', email_responsavel='', \
        responsavel_revisao='', responsavel_publicacao='', monitorada=0, \
        data_inicial='1970/01/01', data_final='2030/01/01', pagina_inicial=None, \
        mostrar_no_menu_esquerdo=0, mostrar_no_menu_direito=0, mostrar_no_menu_superior=0, \
        template=None, in_ZMI=1, REQUEST=None, *args, **kw):
        "Modifica as Propriedades da XSecao"
        self.title = title
        self.responsavel = responsavel
        self.email_responsavel = email_responsavel
        self.responsavel_revisao = responsavel_revisao
        self.responsavel_publicacao = responsavel_publicacao
        self.monitorada = monitorada
        try: self.data_inicial = dt2us(data_inicial)
        except: self.data_inicial = dt2us()
        try: self.data_final = dt2us(data_final)
        except: self.data_final = dt2us('2030/01/01')
        self.mostrar_no_menu_esquerdo = mostrar_no_menu_esquerdo
        self.mostrar_no_menu_direito = mostrar_no_menu_direito
        self.mostrar_no_menu_superior = mostrar_no_menu_superior
        if not pagina_inicial:
                for i in self.objectValues('XPagina'):
                        pagina_inicial = i.getId()
        self.pagina_inicial = pagina_inicial
        self.template = template
        if self.getCatalogPrincipal() is not None and self.default_catalog != self.getCatalogPrincipal():
                self.manage_editCataloger(self.getCatalogPrincipal(), None)
        self.reindex_object()

        if REQUEST is not None:
          if in_ZMI:
            return self.manage_PropertiesForm(REQUEST, management_view='Propriedades',
                manage_tabs_message='A Secao <strong>%s</strong> foi editada.' % self.id
                )
          else:
            return self.manage_ZPropertiesForm(REQUEST, management_view='Propriedades',
                manage_tabs_message='A Secao <strong>%s</strong> foi editada.' % self.id
                )

Globals.default__class_init__(XSecao)


def manage_addXSecao(self, id, title, responsavel='admin', email_responsavel='', \
        responsavel_revisao='', responsavel_publicacao='', monitorada=0, \
        data_inicial=None, data_final=None, pagina_inicial=None, template=None, \
        mostrar_no_menu_direito=0, mostrar_no_menu_esquerdo=0, mostrar_no_menu_superior=0, REQUEST=None):
        """Cria um XSecao e o adiciona ao seu parent Folder.
        O argumento 'self' referencia o parent Folder.
        """
        secao = XSecao(id, title)
        secao.responsavel = responsavel
        secao.email_responsavel = email_responsavel
        secao.responsavel_revisao = responsavel_revisao
        secao.responsavel_publicacao = responsavel_publicacao
        secao.monitorada = monitorada
        try: secao.data_inicial = dt2us(data_inicial)
        except: secao.data_inicial = dt2us()
        try: secao.data_final = dt2us(data_final)
        except: secao.data_final = dt2us('2030/01/01')
        secao.pagina_inicial = pagina_inicial
        secao.template = template
        secao.mostrar_no_menu_esquerdo = mostrar_no_menu_esquerdo
        secao.mostrar_no_menu_direito = mostrar_no_menu_direito
        secao.mostrar_no_menu_superior = mostrar_no_menu_superior
        self._setObject(id, secao)
        secao = self.restrictedTraverse(id)
        if secao.getCatalogPrincipal() is not None:
          secao.manage_editCataloger(secao.getCatalogPrincipal(), None)
          secao.index_object()

        if REQUEST is not None:
                return self.manage_main(self, REQUEST)

manage_addXSecaoForm = Globals.HTMLFile('dtml/addXSecaoForm', globals())
